package w2phtml.xhtml.content;

import static w2phtml.util.Misc.*;

import java.util.LinkedList;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import w2phtml.office.XMLString;
import pro.litvinovg.xml.Debug;
import static w2phtml.office.XMLString.*;
import w2phtml.xhtml.Converter;
import w2phtml.xhtml.XhtmlConfig;

//LinkedList<String>   stringList = new LinkedList<String>();
public class Separator {

	private static final String COMMENT_END = "</Description>";
	public Node currentComment = null; 
	private static final String NONE = "none";
	private static final String DIV = "div";
	private int splitFilesOutlineLevel = 0;
	private boolean alignSplitToPages;
	private int lastSplitPageNum = 1;
	private Integer pageNumber = null;
	private String breakStyle = null;

	private static LinkedList<Integer> greenstoneStack = new LinkedList<Integer>();
	private int[] headingPath;
	static boolean pageOpened = false;
	// headings none
	private static boolean isGreenstoneCommentsNeeded;
	private static boolean rdfSeparation = false;
	// sections div none
	private static boolean pagination;
	private static Converter converter = null;
	private Node prevPageContainer = null;
	private PageContainer pageContainer = null;
	private static final Logger logger  = LoggerFactory.getLogger(Separator.class);

	public Separator(XhtmlConfig config, Converter converter) {
		Separator.converter = converter;
		isGreenstoneCommentsNeeded = config.getGreenstoneSeparation();
		pagination = config.pagination();
		splitFilesOutlineLevel = config.getXhtmlSplitLevel();
		alignSplitToPages = config.getAlignSplitsToPages();
		breakStyle = config.getPageBreakStyle();
		pageContainer = converter.pageContainer;
		rdfSeparation = converter.isRDF;
		headingPath = new int[10];
	}

	public Node breakOutline(Node currentNode, Node hnode, int pageNum) {
		if (isGreenstoneCommentsNeeded) {
			hnode = addGreenstoneOutlineBreaks(currentNode, hnode, pageNum);
		} else if (rdfSeparation) {
			hnode = addRDFOutlineBreaks(currentNode, hnode, pageNum);
		} else if(isFileSplitNeed(currentNode, pageNum)) {
			hnode = addFileOutlineBreaks(currentNode, hnode, pageNum);
		} else {
			logger.error("Shouldn't be here. No separation method found.");
		}
		return hnode;
	}
	
	private Node addFileOutlineBreaks(Node currentNode, Node hnode, int pageNum) {
		
		prevPageContainer = hnode;	
		hnode = closePage(hnode);
		hnode = splitFiles(hnode);
		lastSplitPageNum = pageNum;
		hnode = openPage(hnode, pageNum);
		return hnode;	
	}

	private Node addGreenstoneOutlineBreaks(Node currentNode, Node hnode, int pageNum) {
		
		hnode = closePage(hnode);
		Integer curLevel = getOutlineLevel(currentNode);
		closeCommentHeadings(hnode, curLevel);
		String title = getOutlineTitle(currentNode);
		openCommentHeading(hnode, title);
		addToCommentStack(curLevel.toString());
		hnode = openPage(hnode, pageNum);
		return hnode;	
		}
	
	public Node addRDFOutlineBreaks(Node currentNode, Node hnode, int pageNum) {
		 
		prevPageContainer = hnode;	
		hnode = closePage(hnode);
		Integer curLevel = getOutlineLevel(currentNode);
		updateHeadingPath(curLevel);
		hnode = splitFiles(hnode);
		setContentPath(hnode,curLevel);
		String title = getOutlineTitle(currentNode);
		setContentName(hnode,title);
		hnode = openPage(hnode, pageNum);
		return hnode;
	}
	
	public static boolean isNotOutlineNode(Node currentNode) {
		Integer curLevel = getOutlineLevel(currentNode);
		if (curLevel < 1) {
			return true;
		}
		String title = getOutlineTitle(currentNode);		
		if (title == null || title.isEmpty()) {
			return true;
		}
		return false;
	}

	private static Node getRealOntlineNode(Node currentNode) {
		Node outlineNode = null;
		if (currentNode.getNodeName().contentEquals(TABLE_TABLE)) {
			outlineNode = getOulineNodeFromTable(currentNode);			
		} else {
			outlineNode = currentNode;
		}
		return outlineNode;
	}

	private static int getOutlineLevel(Node currentNode) {
		Node outlineNode = getRealOntlineNode(currentNode);
		String levelAttr = getAttribute(outlineNode, XMLString.TEXT_OUTLINE_LEVEL);
		if (levelAttr == null || levelAttr.isEmpty()) {
			return 0;
		} else {
			return Integer.parseInt(levelAttr);
		}
	}
		
	private static Node getOulineNodeFromTable(Node currentNode) {
		NodeList tableNodes = currentNode.getChildNodes();
		int i = 0;
		while (i < tableNodes.getLength()) {
			Node tableNode = tableNodes.item(i++);
			if (tableNode.getNodeName().contentEquals(TABLE_TABLE_ROW)) {
				NodeList rowNodes = tableNode.getChildNodes();
				int j = 0;
				while (j < rowNodes.getLength()) {
					Node rowNode = rowNodes.item(j++);
					if (rowNode.getNodeName().contentEquals(TABLE_TABLE_CELL)) {
						NodeList cellNodes = rowNode.getChildNodes();
						int k = 0;
						while (k < cellNodes.getLength()) {
							Node cellNode = cellNodes.item(k++);
							String levelAttr = getAttribute(cellNode, XMLString.TEXT_OUTLINE_LEVEL);
							if (levelAttr != null && !levelAttr.isEmpty()) {
								String title = getOutlineTitle(cellNode);
								if (!title.isEmpty()) {
									return cellNode;
								}
							}
							
						}
					}
				}
			}
		}
		return currentNode;
	}

	private void setContentName(Node hnode, String title) {
		((Element) hnode).setAttribute("name", title);
		
	}

	public boolean configuredOutlineSplits() {
		if (isFilesSplitLevelSet() ) {
			return true;
		}
		if (isGreenstoneCommentsNeeded) {
			return true;
		}
		if (rdfSeparation) {
			return true;
		}
		return false;
	}

	private boolean isFilesSplitLevelSet() {
		return splitFilesOutlineLevel > 0;
	}
	
	boolean needPagination() {
		if (pagination) {
			return true;
		}
		return false;
	}
	

	private void addToCommentStack(String sLevel) {
		greenstoneStack.offerFirst(Integer.parseInt(sLevel));
	}

	private void updateHeadingPath(int level) {
		if (level == 0) {
			return;
		}
		for (int i = 9; i > level - 1; i--) {
			headingPath[i] = 0;
		}
		headingPath[level - 1]++;
	}

	private Node splitFiles(Node hnode) {
		hnode = converter.nextOutFile();
		
		return hnode;
	}

	private void setContentPath(Node hnode, int curLevel) {
		StringBuilder path = new StringBuilder();
		for (int i = 0; i < curLevel; i++) {
			if (path.length() != 0 ) {
				path.append(" ");
			}
			path.append(headingPath[i]);
		}
		((Element) hnode).setAttribute("path", path.toString());
	}

	boolean needOutlineSplitOnThisPage(Node currentNode, int curPage) {
		if (!configuredOutlineSplits()) {
			return false;
		}
		if (isNotOutlineNode(currentNode)) { 
			return false;
		}
		if (isFilesSplitEnabled() && !isFileSplitNeed( currentNode, curPage)) {
			return false;
		}
		return true;
	}

	private boolean isFilesSplitEnabled() {
		return splitFilesOutlineLevel != 0;
	}
	
	private boolean isFileSplitNeed(Node currentNode, int pageNum) {
		boolean result = false;
		Integer curLevel = getOutlineLevel(currentNode);
		if ( splitFilesOutlineLevel >= curLevel ) {
			if (alignSplitToPages) {
				if (lastSplitPageNum != pageNum) {
					result = true;
				}	
			} else {
				result = true;
			}
		}
		return result;
	}
	
	public boolean isPageOpened() {
		return pageOpened;
	}
	public Node turnPage(Node hnode, int pageNum){
		if (needPagination()) {
			if (pageOpened) {
				hnode = closePage(hnode);
			}
			hnode = openPage(hnode, pageNum);
		}
		return hnode;
	}
	/**
	 * Opens main document section heading tag
	 */
	public Node startDocumentSeparation(Node hnode, String title, int pageNum) {
		if (!isGreenstoneCommentsNeeded && !needPagination()) {
			return hnode;
		}
		if (isGreenstoneCommentsNeeded) {
			// Create global section
			openCommentHeading(hnode, title);
		}
		hnode = openPage(hnode, pageNum);

		return hnode;
	}

	// Method to close open tags at the end of the document
	public Node endDocument(Node hnode) {
		if (!isGreenstoneCommentsNeeded && !needPagination()) {
			return hnode;
		}
		if (pageOpened) {
			hnode = closePage(hnode);
		}
		if (isGreenstoneCommentsNeeded) {
			closeCommentHeadings(hnode, 0);
			// Close global section
			addCloseComment(hnode);
		}

		return hnode;
	}

	private Node openPageDiv(Node node, int curPageNum) {
		if (node == null) {
			System.out.println("Error: node is null on openPageDiv");
			return node;
		}
		if (prevPageContainer != null && alignSplitToPages) {
			alignFilesByHeadings(node, curPageNum);
		}
		insertPageBreak(node, curPageNum);
		Document doc = node.getOwnerDocument();
		Element openBlock = (Element) doc.createElement(DIV);
		openBlock.setAttribute("class", "pageNum");
		openBlock.setAttribute("page", Integer.toString(curPageNum));
		// insert open section comment before header node
		node.appendChild((Node) openBlock);
		node = openBlock;
		return openBlock;
	}

	private void insertPageBreak(Node node, int curPageNum) {
		if (pageNumber != null && pageNumber != curPageNum) {
			Document doc = node.getOwnerDocument();
			Element pageBreak = (Element) doc.createElement(DIV);
			pageBreak.setAttribute("class", "pageBreak");
			//Prevent from collapsing on export to xml
			pageBreak.setTextContent(" ");
			if (breakStyleIsSet()) {
				applyBreakStyle(pageBreak);
			}
			
			node.appendChild(pageBreak);
		}
		pageNumber = curPageNum;
	}

	private void applyBreakStyle(Element pageBreak) {
		pageBreak.setAttribute("style", breakStyle);
	}

	public boolean breakStyleIsSet() {
		if (breakStyle == null || breakStyle.isEmpty()) {
			return false;
		} else {
			return true;
		}
	}

	private void alignFilesByHeadings(Node node, int pageNum) {
		Document newdoc = node.getOwnerDocument();
		Node prevPage = prevPageContainer.getParentNode();
		String prevPageNum = getAttribute(prevPage, "page");
		if (prevPageNum != null && prevPageNum.equals(Integer.toString(pageNum))) {
			if (isElement(prevPage, DIV)) {
				Node importedNode = newdoc.importNode(prevPage, true);
				node.appendChild(importedNode);
				Node prevDoc = prevPage.getParentNode();
				if (prevDoc != null) {
					prevDoc.removeChild(prevPage);
				}
			}
		}
		// no more arrange needed till next file separation
		prevPageContainer = null;

	}

	private static Node exitPageDiv(Node node) {

		while (!isRoot(node) && !isElement(node, DIV)) {
			node = node.getParentNode();
		}
		Node result = node.getParentNode();
		if (node.getChildNodes().getLength() == 1) {
			if (node.getFirstChild().getChildNodes().getLength() == 0) {
				result.removeChild(node);
			}
		}

		return result;
	}

	public static String getOutlineTitle(Node currentNode) {
		Node outlineNode = getRealOntlineNode(currentNode);
		Node content = outlineNode.cloneNode(true);
		NodeList lineBreaks = ((Element) content).getElementsByTagName(XMLString.TEXT_LINE_BREAK);
		NodeList textTabs = ((Element) content).getElementsByTagName(XMLString.TEXT_TAB);
		replaceWithSpaces(lineBreaks);
		replaceWithSpaces(textTabs);
		deleteNotesIn(content);
		deleteCommentsIn(content);
		String title = content.getTextContent().trim();
		return title;
	}

	

	private static void deleteCommentsIn(Node content) {
		NodeList notes = ((Element) content).getElementsByTagName(XMLString.OFFICE_ANNOTATION);
		int j = 0;
		while (j < notes.getLength()) {
			Node note = notes.item(j);
			note.getParentNode().removeChild(note);
		}
	}
	
	private static void deleteNotesIn(Node content) {
		NodeList notes = ((Element) content).getElementsByTagName(XMLString.TEXT_NOTE);
		int j = 0;
		while (j < notes.getLength()) {
			Node note = notes.item(j);
			note.getParentNode().removeChild(note);
		}
	}

	private static void replaceWithSpaces(NodeList contentNodes) {
		int i = 0;
		while (i < contentNodes.getLength()) {
			Node node = contentNodes.item(i);
			Document doc = node.getOwnerDocument();
			Node testSpace = doc.createTextNode(" ");
			node.getParentNode().insertBefore(testSpace, node);
			node.getParentNode().removeChild(node);
			i++;
		}
	}

	private void openCommentHeading(Node hnode, String title) {
		Document doc = hnode.getOwnerDocument();
		Node openSection = doc.createComment(openHeadingCommentText(title));
		// insert open section comment before header node
		this.currentComment = openSection;
		hnode.appendChild(openSection);
	}

	private static void addCloseComment(Node node) {
		Document doc = node.getOwnerDocument();
		Node closeSection = doc.createComment("</Section>");
		// insert open section comment before header node
		node.appendChild(closeSection);
	}

	private static void closeCommentHeadings(Node hnode, int nLevel) {
		if (greenstoneStack.isEmpty()) {
			return;
		}
		// Close all sections with level less than current
		while (nLevel <= greenstoneStack.peek()) {
			addCloseComment(hnode);
			greenstoneStack.poll();
			if (greenstoneStack.isEmpty()) {
				break;
			}
		}
	}

	public Node closePage(Node hnode) {
		if (pageOpened == false || !needPagination()) {
			return hnode;
		}
		hnode = exitPageContainer(hnode);
		if (needPagination()) {
			hnode = exitPageDiv(hnode);
		}
		pageOpened = false;
		return hnode;
	}

	public Node openPage(Node hnode, Integer pageNum) {
		if (pageOpened == true || !needPagination()) {
			return hnode;
		}
		hnode = openPageDiv(hnode, pageNum);
		pageOpened = true;
		hnode = enterPageContainer(hnode);
		return hnode;
	}

	private Node exitPageContainer(Node hnode) {
		String className = ((Element) hnode).getAttribute("class");
		if (!className.equals("pageContainer")) {
			logger.error("Can't exit not my container!");
			Debug.printNode(hnode);
			Debug.printStackTrace();
			System.exit(1);
		}
		Element parentNode = (Element) hnode.getParentNode();
		return parentNode;
	}

	private Node enterPageContainer(Node hnode) {
		if (hnode == null) {
			System.out.println("Enter page container error. hnode is null");
			Debug.printStackTrace();
			System.exit(1);
		}
		Element container = converter.createElement("div");
		container.setAttribute("class", "pageContainer");
		if (!pageContainer.getCurrentStyle().isEmpty()) {
			container.setAttribute("style", pageContainer.getCurrentStyle());
		}
		hnode.appendChild(container);
		return container;
	}

	private static String openHeadingCommentText(String title) {
		String comment = "<Section>\n<Description>\n<Metadata name=\"Title\">" + title + "</Metadata>\n" + COMMENT_END;
		return comment;
	}
	
	public void appendMetadata(String name, String value) {
		if (!isGreenstoneCommentsNeeded) {
			return;
		}
		if (currentComment == null) {
			System.out.println("Error. No current Comment");
			return;
		}
		String content = currentComment.getTextContent();
		System.out.println(content);
		if (content.endsWith(COMMENT_END)) {
			String firstPart = content.substring(0, content.length()- COMMENT_END.length());
			String metadata = "<Metadata name=\"" + name + "\">" + value + "</Metadata>\n";
			currentComment.setTextContent(firstPart + metadata + COMMENT_END);
			System.out.println("Comment updated.");
		} else {
			System.out.println("Error. Comment doesn't end with " + COMMENT_END);
		}
	}

}
