/************************************************************************
 *
 *  RowStyleConverter.java
 *
 *  Copyright: 2002-2008 by Henrik Just
 *
 *  This file is part of Writer2LaTeX.
 *  
 *  Writer2LaTeX is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Writer2LaTeX is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Writer2LaTeX.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Version 1.0 (2008-11-22)
 *
 */

package w2phtml.xhtml.style;

import w2phtml.office.OfficeReader;
import w2phtml.office.OfficeStyleFamily;
import w2phtml.office.StyleWithProperties;
import w2phtml.util.CSVList;
import w2phtml.xhtml.Converter;
import w2phtml.xhtml.XhtmlConfig;
import w2phtml.xhtml.XhtmlStyleMap;
import w2phtml.xhtml.style.properties.Properties;

/**
 * This class converts OpenDocument row styles to CSS2 styles.
 * Rows formatting includes <em>background</em>, and also <em>height</em>,
 * which is considered elsewhere.
 */
public class RowStyleParser extends StyleWithPropertiesParser {

    /** Create a new <code>RowStyleConverter</code>
     *  @param ofr an <code>OfficeReader</code> to read style information from
     *  @param config the configuration to use
     *  @param converter the main <code>Converter</code> class
     *  @param nType the type of xhtml to use
     */
    public RowStyleParser(OfficeReader ofr, XhtmlConfig config, Converter converter, int nType) {
        super(ofr,config,converter,nType);
        // Style maps for rows are currently not supported.
        this.styleMap = new XhtmlStyleMap();
    }

    /** Get the family of row styles
     *  @return the style family
     */
    public OfficeStyleFamily getStyles() {
        return ofr.getRowStyles();
    }
	
    /** Create default tag name to represent a row object
     *  @param style to use
     *  @return the tag name.
     */
    public String getStyleTag() {
        return "tr";
    }
	
    /** Convert formatting properties for a specific Row style.
     *  @param style the style to convert
     *  @param props the <code>CSVList</code> object to add information to
     *  @param bInherit true if properties should be inherited from parent style(s)
     */
    public void applyProperties(StyleWithProperties style, Properties props, boolean bInherit) {
        getFrameSP().cssBackground(style,props,bInherit);
    }
	
}
